/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.openjpa.persistence.query;

/**
 * Utility for converting elements to JPQL string fragments.
 * 
 * @author Pinaki Poddar
 *
 */
public class JPQLHelper {
	private static final StringBuffer SINGLE_QUOTE = new StringBuffer("'");
	
	static String toJPQL(AliasContext ctx, Object o) {
		if (o == null)
			return "NULL";
		if (o instanceof Visitable)
			return ((Visitable)o).asExpression(ctx);
		if (o instanceof Class)
			return ctx.getEntityName((Class)o);
		if (o instanceof String)
			return SINGLE_QUOTE + (String)o + SINGLE_QUOTE;
		return o.toString();
	}
}
